#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from db_paribhasha import credentials
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.metadata_patterns import MetadataPatterns
from db_paribhasha.error_handler import OperationNotPerformedError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if MetadataPatterns.exists():
        raise OperationNotPerformedError("Table already exists")
    MetadataPatterns.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def create_metadata_pattern(pattern_type, use_count, pattern, info, user_id):
    """
    Creates a new metadata pattern for a user ID.

    Parameters
    ----------
    pattern_type: string
        Pattern type: Global or User based pattern
    use_count : Number
        Usage of the pattern till now
    pattern : String
        The pattern to save
    info : Dict
        Info corresponding to the pattern
    user_id: String
        user user_id

    Returns
    -------
    user_profile: pynamo object
        Pynamo object containing the values
    """

    created_ts = int(round(time.time() * 1000))
    calculated_id = pattern_type + '-' + str(created_ts)
    user_profile = MetadataPatterns(
        calculated_id=calculated_id,
        created_ts=created_ts,
        use_count=use_count,
        pattern=pattern,
        info=info,
        user_id=user_id,
        modified_ts=created_ts)
    db_helpers.save_dynamo_object(user_profile)
    return user_profile


def get_metadata_pattern_detail(user_id, pattern_id):
    """
    Gets the detail corresponding to the metadata pattern

    Parameters
    ----------
    user_profile_id : String
        Unique profile ID

    Returns
    -------
    metadata_pattern: pynamo_object
        Pynamo Object containing the pattern detail
    """
    metadata_pattern = MetadataPatterns.get(user_id, pattern_id)
    return metadata_pattern


def get_all_user_patterns(user_id):
    """
    Returns all user patterns for a given User ID

    Parameters
    ----------
    user_id : String/Int
        User ID

    Returns
    -------
    pynamo_object
        Pynamo Object containing the query result
    """

    return MetadataPatterns.query(str(user_id))


def get_count_user_patterns(user_id):
    """
    Returns the count of user patterns existing for the given User ID

    Parameters
    ----------
    user_id : String/Int
        User ID

    Returns
    -------
    Integer
        Count of patterns
    """

    return MetadataPatterns.count(str(user_id))


def get_pattern_data(user_id, pattern):
    """
    Returns data for patterns matching to given User ID

    Parameters
    ----------
    user_id : String/Int
        User ID
    pattern : String
        The pattern to match

    Returns
    -------
    pynamo iterator object
        Iterator containing filter response
    """

    return MetadataPatterns.query(
        str(user_id), MetadataPatterns.pattern == str(pattern))


def get_matching_patterns_count(user_id, pattern):
    """
    Returns matching patterns count

    Parameters
    ----------
    user_id : String/Int
        User ID
    pattern : String
        Pattern to match

    Returns
    -------
    Integer
        Count of matching existing patterns
    """

    return MetadataPatterns.count(
        str(user_id), MetadataPatterns.pattern == str(pattern))
